use crate::node::Node;

#[derive(Debug)]
pub struct Set<T: Ord> {
    root: Option<Box<Node<T>>>,
}

impl<T: Ord> Default for Set<T> {
    fn default() -> Self {
        Set { root: None }
    }
}

impl<T: Ord> Set<T> {
    pub fn new() -> Self {
        Default::default()
    }

    pub fn insert(&mut self, value: T) -> bool {
        if let Some(ref mut node) = self.root {
            node.insert(value)
        } else {
            self.root = Some(Box::new(Node::new(value)));
            true
        }
    }

    pub fn count(&self) -> i32 {
        self.root.as_ref().map_or(0, |n| n.count())
    }

    pub fn count_lt(&self, value: &T) -> i32 {
        self.count() - self.count_ge(value)
    }

    pub fn count_le(&self, value: &T) -> i32 {
        self.root.as_ref().map_or(0, |n| n.count_le(value))
    }

    pub fn count_ge(&self, value: &T) -> i32 {
        self.root.as_ref().map_or(0, |n| n.count_ge(value))
    }

    pub fn count_gt(&self, value: &T) -> i32 {
        self.count() - self.count_le(value)
    }

    pub fn lower_bound(&self, value: &T) -> Option<&T> {
        self.root.as_ref().and_then(|n| n.lower_bound(value))
    }
}
